// OzAPI.cpp : Defines the class behaviors for the application.
//


#include <string.h>
#include <stdlib.h>

#include "TkOzAPI.h"
#include "OzAPI.h"


// globals

char* ozResult = NULL;
char tclCmd[100000];

// local

static char tclWidget[20];


// for menus


#define MAX_ITEM	20
#define	MAX_PULLDOWN	10

static int	menuID[MAX_PULLDOWN+1][MAX_ITEM+1];
static int	popupID[MAX_ITEM+1];
static int	numPullDown;
static int	numItem[MAX_PULLDOWN+1];
static int	numPopup;


//////////////// misc //////////////////////


// Handle GNU fortran string parameter. The compiler adds a length parameter for each.
// Copy to scratch string (MAX_STRING long )and remove trailing blanks.
// Also used for data of file.

void GnuParam (char* outStr, char* inStr, int length)
{
	char c;
	int out = 0;
	int this = 0;

	while (this < length && inStr[this] == ' ')
		this++;

	while (this < length && out < MAX_STRING-2)
	{
		c = inStr[this++];
		if (c == '[')  outStr[out++] = '\\';  // chars that foul up Tcl
		if (c == ']')  outStr[out++] = '\\';
		if (c == '$')  outStr[out++] = '\\';
		outStr[out++] = c;
	}
	outStr[out] = '\0';

}

// use param as is with leading blanks

void GnuParamAsIs (char* outStr, char* inStr, int length)
{
	char c;
	int out = 0;
	int this = 0;


	while (this < length && out < MAX_STRING-2)
	{
		c = inStr[this++];
		if (c == '[')  outStr[out++] = '\\';  // chars that foul up Tcl
		if (c == ']')  outStr[out++] = '\\';
		if (c == '$')  outStr[out++] = '\\';
		outStr[out++] = c;
	}
	outStr[out] = '\0';

}


//////////////// Errors ////////////////////

void OzFatalError (char* msg)
{
	fprintf (stderr, "Fatal Error: %s\n", msg);
	exit(-1);
}

////////////////// check Tcl command ////////////////////////////////

void TclCheck(int tclError)
{
	if (tclError != TCL_OK)
	{
		OzFatalError(Et_Interp->result);
	}
}

//////////////// return result from tcl command into global ///////////



/////////////////// MENUS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

static	void	InitMenu()
{
	int	i;
	int	j;
	for (i=0; i<MAX_PULLDOWN+1; i++)
	{
		numItem[i] = 0;
		for (j=0; j<MAX_ITEM+1; j++)
		{
			menuID[i][j] = 0;
			popupID[j] = 0;
		}
	}
	numPullDown = 0;
	numPopup = 0;
}

long ozaddmenuitem_ (int* pullDown, char* string, int* id, int lString)
{
	char str[MAX_STRING];

	int	pd = *pullDown;
	int	item = numItem[pd];

	GnuParam(str, string, lString); // parameter

	item++;
	sprintf(tclWidget, ".mb.%d.m", pd);
	Et_EvalF(Et_Interp, 
	"%s add command -label \"%q\" -command {OzMenu %d %d %d}", 
		tclWidget, str, pd, item, *id);

	numItem[pd] = item;
	return (item);
}

 long ozaddpulldown_ (char* string, int lString)
{
	int ret = 88;
	char str[MAX_STRING];
	GnuParam(str, string, lString); // parameter

	if (numPullDown >= MAX_PULLDOWN)
		OzFatalError ("ozaddpulldown - too many pulldowns");
	numPullDown++;
	sprintf(tclWidget, ".mb.%d", numPullDown);
	Et_EvalF(Et_Interp, "menubutton %s -text \"%s\" -menu .mb.%d.m", 
		tclWidget, str, numPullDown);
	if (strcmp(str, "Help") == SAME)
		Et_EvalF(Et_Interp, "pack %s -side right", tclWidget);
	else
		Et_EvalF(Et_Interp, "pack %s -side left", tclWidget);
	Et_EvalF(Et_Interp, "menu %s.m", tclWidget);


	ret = numPullDown;
	return (ret);
}


 void ozclearpulldown_ (int* pullDown)
{

	int	pd = *pullDown;
	int	item;
	int 	i;

	if (pd > numPullDown || pd < 1)
		OzFatalError ("ozclearpulldown - illegal pulldown");
	item = numItem[pd];

	for (i=1; i<=numPullDown; i++)
	{
		sprintf(tclWidget, ".mb.%d", i);
		Et_EvalF(Et_Interp, "destroy %s", tclWidget);
	}
	numItem[pd] = 0;
}


 void ozclearallmenus_ ()
{
	int		pd;

	for (pd=numPullDown; pd>0; pd--)
	{
		ozclearpulldown_(&pd);
	}
	InitMenu();
}




 int ozaddpopupitem_ (char* string, int* id, int lString)
{
	char str[MAX_STRING];
	GnuParam(str, string, lString); // parameter

	if (numPopup == 0)
	{
		Et_EvalF(Et_Interp, "menu .pm"); 
	}

	if (numPopup >= MAX_ITEM)
		OzFatalError ("ozaddpopupitem - too many items");
	numPopup++;
	Et_EvalF(Et_Interp, ".pm add command  -label \"%s\" "
		"-command {OzMenu %d %d %d}", 
		str, 0, numPopup, *id);
	return (numPopup);
}
 


 void ozclearpopupmenu_ ()
{
	Et_EvalF(Et_Interp, "catch {destroy .pm}"); 

	numPopup = 0;

}

void ozdisplaypopup_ ()

{
 	Et_EvalF(Et_Interp, "tk_popup .pm  200 200"); 

}




//////////////////////// Message boxes  ////////////////////////





 long ozmessagebox_ (char* string, int* type, int* icon, 
	int* button, int lString)
{
	char 	str[MAX_STRING];
	char 	buttonLabels[MAX_STRING];
	char 	bitmapName[MAX_STRING];
	int	defaultButton;
	long	ret;
	int iResult;

	GnuParam(str, string, lString); // parameter

	if (*type == 1)
		strcpy(buttonLabels, "Abort Retry Ignore");
	else if (*type == 2)
		strcpy(buttonLabels, "OK");
	else if (*type == 3)
		strcpy(buttonLabels, "OK Cancel");
	else if (*type == 4)
		strcpy(buttonLabels, "Retry Cancel");
	else if (*type == 5)
		strcpy(buttonLabels, "Yes No");
	else if (*type == 6)
		strcpy(buttonLabels, "Yes No Cancel");
	else
		OzFatalError ("ozmessagebox - invalid box type");

	if (*icon == 1)
		strcpy(bitmapName, "warning");
	else if (*icon == 2)
		strcpy(bitmapName, "info");
	else if (*icon == 3)
		strcpy(bitmapName, "question");
	else if (*icon == 4)
		strcpy(bitmapName, "error");
	else
		OzFatalError ("ozmessagebox - invalid icon type");

	if (*button == 1)
		defaultButton = 0;
	else if (*button == 2)
		defaultButton = 1;
	else if (*button == 3)
		defaultButton = 2;
	else
		OzFatalError ("ozmessagebox - invalid default button");

	Et_EvalF(Et_Interp, "set ozResult [ tk_dialog .msg \"Message\" \"%s\" %s %d %s ]", 
		str, bitmapName, defaultButton, buttonLabels);
	Et_EvalF(Et_Interp, "OzResults $ozResult"); 


	iResult = atoi(ozResult);

	// return button code

	if (*type == 1)
	{
		if (iResult == 0)
			ret = 1;
		else if (iResult == 1)
			ret = 6;
		else
			ret = 3;
	}
	else if (*type == 2)
	{
			ret = 5;
	}
	else if (*type == 3)
	{
		if (iResult == 0)
			ret = 5;
		else
			ret = 2;
	}
	else if (*type == 4)
	{
		if (iResult == 0)
			ret = 6;
		else
			ret = 2;
	}
	else if (*type == 5)
	{
		if (iResult == 0)
			ret = 7;
		else
			ret = 4;
	}
	else if (*type == 6)
	{
		if (iResult == 0)
			ret = 7;
		else if (iResult == 1)
			ret = 4;
		else
			ret = 2;
	}


	return (ret);
}


//////////////////////  window title /////////////////////

 void oztitle_ (char* string, int lString)
{
	char str[MAX_STRING];
	GnuParam(str, string, lString); // parameter
	Et_EvalF(Et_Interp, "wm title . \"%s\" ", str); 
}

/////////////// about //////////////////////////
void ozabout_ ()
{
}

/////////////// status line /////////////////////////////////

 void ozstatus_ (char* string, int lString)
{
	char str[MAX_STRING];
	GnuParam(str, string, lString); // parameter
	Et_EvalF(Et_Interp, ".sf.l config -text \"%s\" ", str); 
}



/////////////// Common dialogs ////////////////////////////////////////

  bool ozcolourdlg_ (float* red, float* green, float* blue )
{

	 bool	ret = FALSE;


	return (ret);
}


// File common dialogs - changed from WINDOZE 

  bool ozfilename_ ( bool* save, char* fileName,  
		char* dlgTitle, char* defExt, char* initialDir,
		int* numPattern, char* filterName, char* filterPattern, 
		int fileNameSize, int dlgTitleSize, int defExtSize, 
		int initialDirSize, int filterNameSize, int filterPatternSize)
{
	char str[MAX_STRING];
	bool ret;
	int i, l;

	if (*save)
		strcpy(tclCmd, "set ozResult [tk_getSaveFile ");
	else
		strcpy(tclCmd, "set ozResult [tk_getOpenFile ");

	strcat (tclCmd, " -defaultextension ");
	GnuParam (str, defExt, defExtSize);
	strcat(tclCmd, str);

	strcat(tclCmd, " -filetypes { ");
	for (i=0; i<*numPattern; i++)
	{
		strcat(tclCmd, " { {");
		GnuParam(str, &filterName[i*filterNameSize], filterNameSize);
		strcat(tclCmd, str);
		strcat(tclCmd, "} {");
		GnuParam(str, &filterPattern[i*filterPatternSize], filterPatternSize);
		strcat(tclCmd, str);
		strcat(tclCmd, "} }");
	}
	strcat(tclCmd, " }");

	strcat (tclCmd, " -initialdir ");
	GnuParam (str, initialDir, initialDirSize);
	strcat(tclCmd, str);

	GnuParam (str, fileName, fileNameSize);
	if (strlen(str) > 0)
	{
		strcat (tclCmd, " -initialfile ");
		strcat (tclCmd, "\"");
		strcat(tclCmd, str);
		strcat (tclCmd, "\"");
	}


	strcat (tclCmd, " -title \"");
	GnuParam (str, dlgTitle, dlgTitleSize);
	strcat(tclCmd, str);
	strcat (tclCmd, "\"");


	strcat (tclCmd, " ]");

	#ifdef DEBUG
	fprintf (stderr, "%s\n", tclCmd);
	#endif

	Et_EvalF(Et_Interp, "%s", tclCmd); 
	Et_EvalF(Et_Interp, "OzResults $ozResult"); 

	l = strlen(ozResult);
	if (l > 0)
	{
		ret = TRUE;
		strcpy(fileName, ozResult);
		for (i=l; i<fileNameSize; i++)
			fileName[i] = ' ';
	}
	else
	{
		ret = FALSE;
		strcpy(fileName, "");
	}

	return (ret);
}


/////////////// Busy cursor ///////////////////////////////////

bool	isBusy = FALSE;

void ozbusy_ ()
{
	isBusy = TRUE;
	Et_EvalF(Et_Interp, ". configure -cursor watch" ); 
}

void oznotbusy_ ()
{
	isBusy = FALSE;
	Et_EvalF(Et_Interp, ". configure -cursor arrow" ); 
}

//////////////// help ////////////////////

void ozhelpcontents_ (char* fileName)
{
 }


void ozhelpindex_ (char* fileName)
{
 
}


